package show;

# This file is part of sqlsus
#
# Copyright (c) 2008,2009 sativouf
# http://sqlsus.sf.net/
#
# Licensed under the GPL version 2.0 license.
# See LICENSE file or
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.html

use strict;
use warnings;

##################################
sub show_database {
	my $db = shift;
	$db = $main::database unless $db;

	print "\n<( $db )>\n";
	my @tabcounts = &db::query("SELECT DISTINCT table_name,count FROM databases WHERE database_name = ? AND table_name != ''", $db);
	while (my $table = shift @tabcounts) {
		my $count = shift @tabcounts;
		if ($count) {
			print "\n\t[$table : $count]\n";
		} else {
			print "\n\t[$table]\n";
		}
		for my $col (&db::query("SELECT column_name FROM databases WHERE database_name = ? and table_name = ?", $db, $table)) {
			print "\t\t$col\n";
		}
	}
}

##################################
sub usage {
	print <<EOF;
Usage : show item

This command shows items already fetched via "get" or "start"

Possible items :
target               : show the target variables
privs                : show the privileges associated with the user
db [db_name]         : show the specified db structure (current if none specified)
databases            : show all the databases
all                  : same as show target + privs + databases
tables               : show the tables of the current database
columns table_name   : show the columns of the current database for the specific table
EOF
}

##################################
sub command {
	my $args = shift;
	my (@header, @contents);

	if ($args eq "") {
		&usage;
		return 0;
	}

	for ($args) {
		if (/^tables|columns/) {
			if (not defined $main::database) {
				print "Error : target database not defined, use \"start\" first\n";
				return 0;
			}
		}

		if (/^target$/i) {
			@header = ("Variable", "Value");
			@contents = map { $_, $main::target{$_} } sort keys %main::target ;
		}
		elsif (/^priv(?:ilege)?s(?:\s+(.*))?$/i) {
			if (not defined $1 or $1 =~ /^(user|)$/i) {
				@header = ("GRANTEE", "PRIVILEGE_TYPE");
				@contents = &db::query("SELECT grantee,privilege_type FROM privileges WHERE grantee = ?", $main::target{user});
			}
		}
		elsif (/^(?:database|db)(?:\s+(.*))?$/i) {
			&show_database($1);
			return 1;
		}
		elsif (/^databases$/i) {
			@header = ("Databases");
			@contents = &db::query("SELECT DISTINCT database_name FROM databases");
		}
		elsif (/^tables$/i) {
			@header = ("Tables in $main::database");
			@contents = &db::query("SELECT DISTINCT table_name FROM databases WHERE database_name = ?", $main::database);
		}
		elsif (/^columns (.*)$/i) {
			my $table = $1;
			@header = ("Columns in $table");
			@contents = &db::query("SELECT column_name FROM databases WHERE database_name = ? and table_name = ?", $main::database, $table);
		}
		elsif (/^all$/i) {
			for my $arg (("target", "privileges user", "databases")) {
				my $padding = int((30 - length($arg)) / 2);
				printf "\n" . "="x$padding . "[ $arg ]" . "="x$padding . "\n";
				&command($arg);
			}
			return 1;
		}
		else { &usage; return 0 }
	}
	&functions::print_array_in_a_box(undef, \@header, @contents);
	return 1;
}

1
